/* game.h - WinGL class definition (for C++)
 * version 0.11e
 * Copyright (C)1995-97 A.Koizuka, alty / Bio_100%  All Rights Reserved.
 * $Header: /WinGL/GAME.H 21    97/01/22 22:30 Koizuka $
 */

#if !defined(_INC_GAME)
# define _INC_GAME

#if !defined(_INC_WINGL)
# include "WinGL.h"
#endif
#include <string.h>

#if !defined(__MM_SYSTEM_H) && !defined(_INC_MMSYSTEM)
# include <mmsystem.h>
#endif

#if defined(WINGL32) && !defined(EXPORT)
# define EXPORT
#endif
#if defined(WINGL16)
# define MCIDEVICEID WORD
#endif

extern class GApp * MainApp ;
class GWindow;
typedef void (WINGLAPI * IDLEPROC)(LPVOID);

////////////////////////////////////////////////////////////////////////
//                              GApp
//
class GApp {
	static BOOL CALLBACK EXPORT AboutProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM );
	enum { MaxIdle = 8 };
	int currentIdle;
	int numIdle;
	struct IDLES {
		IDLEPROC pOnIdle;
		LPVOID pParamIdle;
	} Idles[MaxIdle];

protected:
	HINSTANCE hInstance;
	HINSTANCE hPrev;
	WPARAM exitCode;
	HACCEL hAccel;
	BOOL isok ;
	GWindow * MainWindow ;

	virtual void OnMainWindowDestroy() ;
	friend class GWindow;
public:

	GApp(HINSTANCE hInst,HINSTANCE hPrv, LPSTR, int ) : hInstance(hInst), hPrev(hPrv), hAccel(0), MainWindow(0),currentIdle(0), numIdle(0) { MainApp = this, WinGLInst = hInst ; }

	BOOL isOk() const { return isok; }
	HINSTANCE getInstance() const { return hInstance; }
	HINSTANCE getPrevInstance() const { return hPrev; }
	HACCEL LoadAccelerators( LPCSTR lpszTableName ) {
		return (hAccel = ::LoadAccelerators( hInstance, lpszTableName )) ;
	}
	HCURSOR LoadCursor( LPCSTR lpszCursorName ) {
		return ::LoadCursor( hInstance, lpszCursorName ) ;
	}
	void SetMainWindow(class GWindow * mainwin) { MainWindow = mainwin; }
	GWindow * GetMainWindow() const { return MainWindow; }
	int GetModuleFileName( LPCSTR lpszFileName, unsigned cbFileName ) const {
		return ::GetModuleFileName(hInstance,(LPSTR)lpszFileName,cbFileName);
	}
	int AboutBox( LPCSTR resID, HWND hwnd ) const {
		return DialogBox(getInstance(), resID, hwnd, (DLGPROC)AboutProc);
	}

	BOOL AddIdle(IDLEPROC proc, LPVOID param);
	BOOL RemoveIdle(IDLEPROC proc, LPVOID param);

	static int Run(HWND hwnd, HACCEL hAccel, int& currentIdle, int& numIdle, IDLES * Idles ) ;
	int MessageLoop() ;
} ;

////////////////////////////////////////////////////////////////////////
//                              GWindow
//
class GWindow {
	HWND hwnd ;
	static class GWindow * creatingWindow ;
	friend class GApp;

protected:
	UINT msg ;
	WPARAM wparam ;
	LPARAM lparam ;
	BOOL bFulldragging;

public:
	HPALETTE hpal;

	GWindow(const char * classname, const char * windowname, DWORD dwStyle, int x, int y, int nWidth, int nHeight, HWND hwndParent, HMENU hmenu, LPVOID lpv)
	 : hwnd(0),hpal(0),bFulldragging(0) {
		creatingWindow = this ;
#if WINGL16
		hwnd = CreateWindow(
#else
# ifndef WS_EX_APPWINDOW
#  define WS_EX_APPWINDOW         0x00040000L
# endif
		hwnd = CreateWindowEx( WS_EX_APPWINDOW,
#endif
			classname, windowname,dwStyle,x,y,nWidth,nHeight,hwndParent,hmenu, 
#if WINGL16
			MainApp->getInstance(), 
#else
			GetModuleHandle(NULL),
#endif
			lpv ) ;
		if ( MainApp && !MainApp->GetMainWindow() )
			MainApp->SetMainWindow(this);
	}
#if WINGL32
	GWindow(DWORD dwExStyle, const char * classname, const char * windowname, DWORD dwStyle, int x, int y, int nWidth, int nHeight,
					HWND hwndParent, HMENU hmenu, LPVOID lpv)
	 : hwnd(0),hpal(0),bFulldragging(0) {
		creatingWindow = this ;
		hwnd = CreateWindowEx( dwExStyle,
			classname, windowname,dwStyle,x,y,nWidth,nHeight,hwndParent,hmenu,GetModuleHandle(NULL),lpv ) ;
	}
#endif

	GWindow() : hwnd(0), hpal(0),bFulldragging(0) {}

	static LRESULT CALLBACK EXPORT AppWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam) ;

	BOOL isOk() const { return hwnd != 0 ; }

	static BOOL Register( const char * classname, HCURSOR hCursor, HICON hIcon, LPCSTR lpszMenuName, HBRUSH hbrBackground, WORD style ) {
		WNDCLASS cls;
		cls.hCursor = hCursor;
		cls.hIcon = hIcon;
		cls.lpszMenuName = lpszMenuName,
		cls.hbrBackground = hbrBackground;
#if WINGL32
		cls.hInstance = GetModuleHandle(NULL);
#else
		cls.hInstance = MainApp->getInstance();
#endif
		cls.style = style;
		cls.lpszClassName = classname;
		cls.cbWndExtra = 4 ;
		cls.cbClsExtra = 0;
		cls.lpfnWndProc = (WNDPROC)AppWndProc ;
#if WINGL16
		return RegisterClass(&cls) ;
#else
		return RegisterClass(&cls) != 0;
#endif
	}

	HDC GetDC() { return ::GetDC(hwnd) ; }
	int ReleaseDC(HDC hdc) { return ::ReleaseDC(hwnd,hdc); }
	BOOL Show(int flag) { return ::ShowWindow(hwnd, flag) ; }
	void Close() { DestroyWindow(hwnd); }

	void ScrollWindow(int dx, int dy, const LPRECT lprcScroll, const LPRECT lprcClip ) {
		::ScrollWindow(hwnd,dx,dy,lprcScroll,lprcClip);
	}
	void SetWindowText( LPCSTR lpsz ) { ::SetWindowText(hwnd,lpsz) ; }
	void InvalidateRect(const LPRECT lprect, BOOL fErase) {
		::InvalidateRect(hwnd,lprect,fErase) ;
	}
	UINT SetTimer(UINT idTimer, UINT uTimeout, TIMERPROC tmprc ) {
		return ::SetTimer(hwnd,idTimer,uTimeout,tmprc );
	}
	BOOL KillTimer(UINT idTimer) { return ::KillTimer(hwnd,idTimer); }

	void ScreenToClient(HWND hwnd,LPPOINT lppoint) { ::ScreenToClient(hwnd,lppoint); }
	void ScreenToClient(LPPOINT lppoint) { ScreenToClient(getHWND(),lppoint); }
	BOOL SetClientSize(unsigned width, unsigned height, BOOL bRepaint = TRUE, BOOL bForce = TRUE);

	HWND getHWND() const { return hwnd; }
	operator HWND() const { return hwnd; }
	HMENU GetMenu() const { return ::GetMenu(hwnd); }
	HMENU GetMenu(HWND hwnd) const { return ::GetMenu(hwnd); }
	HMENU GetSystemMenu(BOOL fRevert) { return ::GetSystemMenu(hwnd,fRevert);}
	LONG GetStyle() const { return ::GetWindowLong(hwnd,GWL_STYLE); }
	void SetStyle(LONG style) {
		SetWindowLong(hwnd,GWL_STYLE,style);
		SetWindowPos( hwnd,HWND_TOP, 0,0,0,0,SWP_FRAMECHANGED|SWP_NOMOVE|SWP_NOSIZE|SWP_NOCOPYBITS );
	}

protected:
	virtual void OnDestroy() ;
	virtual LRESULT OnCommand(short wNotifyCode, short wId, HWND hwndCtl) ;
	virtual void OnActivate(BOOL) ;
	virtual void OnEnable(BOOL) ;
	virtual void OnPaint(HDC hdc) ;
	virtual LRESULT OnSize() ;
	virtual LRESULT OnWindowPosChanging(LPWINDOWPOS) ;
	virtual void OnMaximized() ;
	virtual void OnMinimized() ;
	virtual void OnRestored() ;
	virtual void OnSysRestored() ;
	virtual void OnMinMaxInfo( MINMAXINFO FAR * lpmmi ) ;
	virtual void OnKeyDown( int nVirtKey, DWORD lKeyData ) ;
	virtual void OnKeyUp( int nVirtKey, DWORD lKeyData ) ;
	virtual void OnMenuOpen() ;
	virtual void OnMenuCancel() ;
	virtual void OnSetCursor() ;
	virtual void OnCursorChange() ;
	virtual void OnLButtonDown( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnLButtonUp( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnMButtonDown( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnMButtonUp( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnRButtonDown( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnRButtonUp( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnMouseMove( WPARAM fwKey, WORD x, WORD y ) ;
	virtual void OnEnterSizeMove() ;
	virtual void OnExitSizeMove() ;

	virtual int OnMciAborted(WORD mciDeviceId);
	virtual int OnMciSuccessful(WORD mciDeviceId);
	virtual int OnMciSuperseded(WORD mciDeviceId);
	virtual int OnMciFailure(WORD mciDeviceId);

	void setHWND(HWND hWnd) {
		hwnd = hWnd;
#if WINGL16
		SetWindowLong(hwnd, 0, (LONG)(void far *)this);
#else
		SetWindowLong(hwnd, 0, (LONG)this);
#endif
	}

	virtual LRESULT Dispatch() ;
} ;

inline GWindow * GetGWindow(HWND hwnd) {
	return (GWindow *)GetWindowLong(hwnd,0);
}

////////////////////////////////////////////////////////////////////////
//                              GGameWindow
//
class GGameWindow : public GWindow {
	BOOL PausebyFocus ;
	BOOL PausebyUser ;
	UINT idTimer;
	BOOL bCallOnPause;
	UINT uPeriodTime;
	LONG uPeriodStartTime;
	LONG uPeriodCount;
	LONG uPeriodWMTimer;
	LONG uPeriodNextTime;
	LONG uFrameTime;
	UINT uFrameCount;
	UINT uLastFrameCount;
	BOOL bDirtyCount;
	BOOL bIcon;
	HCURSOR hcsrNormal, hcsrActive;
	static HCURSOR hcsrDefaultNormal, hcsrDefaultActive;
private:
	virtual void DrawTitle();
	static void CALLBACK DispatchPeriod(HWND,UINT,UINT,DWORD);
	static void WINGLAPI DispatchIdle(LPVOID self) ;

protected:
	const char * title;

public:
	static BOOL Register( const char * classname, HCURSOR hnCursor, HCURSOR haCursor, HICON hIcon, LPCSTR lpszMenuName, HBRUSH hbrBackground, WORD style) {
		hcsrDefaultNormal = hnCursor, hcsrDefaultActive = haCursor;
		return GWindow::Register(classname,NULL,hIcon, lpszMenuName,hbrBackground,style);
	}
	static BOOL Register( const char * classname, HCURSOR hCursor, HICON hIcon, LPCSTR lpszMenuName, HBRUSH hbrBackground, WORD style) {
		hcsrDefaultNormal = hcsrDefaultActive = hCursor;
		return GWindow::Register(classname,NULL,hIcon, lpszMenuName,hbrBackground,style);
	}

	void SetPause( BOOL newPause, BOOL bForce = TRUE ) ;
	int GetPause() const {
		return (int)PausebyFocus + (((int)PausebyUser) << 1) ;
	}
	void SetTitle(const char * Title) { title = Title; DrawTitle(); }
	void RestoreActiveCursor() {
		if ( hcsrActive != hcsrDefaultActive ) {
			hcsrActive = hcsrDefaultActive;
			OnSetCursor();
		}
	}
	HCURSOR SetActiveCursor( HCURSOR hc ) {
		HCURSOR last = hcsrActive;
		if ( last != hc ) {
			hcsrActive = hc;
			OnSetCursor();
		}
		return last;
	}
	void RestoreNormalCursor() {
		if ( hcsrNormal != hcsrDefaultNormal ) {
			hcsrNormal = hcsrDefaultNormal;
			OnSetCursor();
		}
	}
	HCURSOR SetNormalCursor( HCURSOR hc ) {
		HCURSOR last = hcsrNormal;
		if ( last != hc ) {
			hcsrNormal = hc;
			OnSetCursor();
		}
		return last;
	}

	BOOL StartPeriod( UINT milisecond, UINT wmTimerPeriod = 200, BOOL callOnPause = FALSE) ;
	void StopPeriod() ;
	UINT GetPeriod() const { return uPeriodTime ; }
	UINT GetFrameCount() const { return uLastFrameCount; }

	GGameWindow( const char * Title, const char * classname, DWORD dwStyle,
					int x, int y, int nWidth, int nHeight,
					HWND hwndParent = 0, HMENU hmenu = 0, LPVOID lpv = 0)
	 : title(Title), idTimer(0), uPeriodTime(0), PausebyFocus(FALSE), PausebyUser(TRUE), bIcon(FALSE), hcsrActive(hcsrDefaultActive),hcsrNormal(hcsrDefaultNormal),
	 GWindow(classname, "", dwStyle,x,y,nWidth,nHeight,hwndParent,hmenu,lpv) {
		if ( isOk() ) {
			SetPause(FALSE, TRUE);
		}
	}
#if WINGL32
	GGameWindow( DWORD dwExStyle, const char * Title, const char * classname, DWORD dwStyle,
					int x, int y, int nWidth, int nHeight,
					HWND hwndParent = 0, HMENU hmenu = 0, LPVOID lpv = 0)
	 : title(Title), idTimer(0), uPeriodTime(0), PausebyFocus(FALSE), PausebyUser(TRUE), bIcon(FALSE), hcsrActive(hcsrDefaultActive),hcsrNormal(hcsrDefaultNormal),
	 GWindow(dwExStyle, classname, "", dwStyle,x,y,nWidth,nHeight,hwndParent,hmenu,lpv) {
		if ( isOk() ) {
			SetPause(FALSE, TRUE);
		}
	}
#endif

protected:
	// override function
	virtual void OnDestroy() ;
	virtual void OnActivate(BOOL) ;
	virtual void OnEnable(BOOL) ;
	virtual LRESULT OnSize() ;
	virtual void OnSysRestored() ;
	virtual void OnMenuOpen() ;
	virtual void OnMenuCancel() ;
	virtual void OnExitSizeMove() ;

	virtual void OnPause(int lastpause);
	virtual void OnSetCursor() ;

	virtual BOOL OnPeriod( HDC hdc, UINT dropCount );
} ;

////////////////////////////////////////////////////////////////////////
//                              GPalette
//
class GPalette {
	HPALETTE hPal ;
public:
	GPalette(HPALETTE hpal = 0) : hPal(hpal) {}
	~GPalette() {
		if ( hPal ) {
			DeleteObject(hPal);
			hPal = 0;
		}
	}
	void set(HPALETTE hpal) { hPal = hpal; }
	operator HPALETTE() const { return hPal; }
} ;

////////////////////////////////////////////////////////////////////////
//                              GMenu
//
class GMenu {
	HMENU hMenu;
public:
	GMenu(HMENU hmenu = 0) : hMenu(hmenu) {}
	BOOL AppendSeparator() {
		return AppendMenu(hMenu, MF_SEPARATOR, 0, "" /*(LPCSTR)NULL*/ );
	}
	BOOL AppendString( UINT idNewItem, LPCSTR lpNewItem, UINT fuFlags = (MF_STRING | MF_ENABLED)) {
		return AppendMenu(hMenu, fuFlags, idNewItem, lpNewItem);
	}
	BOOL CheckItem(UINT item, BOOL bCheck) {
		return CheckMenuItem( hMenu, item, bCheck ? (MF_BYCOMMAND|MF_CHECKED) : (MF_BYCOMMAND|MF_UNCHECKED) ) ;
	}
	BOOL EnableItem(UINT item, BOOL bEnable) {
		return EnableMenuItem( hMenu, item, bEnable ? (MF_BYCOMMAND|MF_ENABLED) : (MF_BYCOMMAND|MF_GRAYED) ) ;
	}
} ;

////////////////////////////////////////////////////////////////////////
//                              GROP
//
class GROP {
	WINGLROP rop;
public:
	GROP() { rop = CreateRop(); }
	~GROP() { DeleteRop(rop); }

	void Transparent(unsigned char tcolor) { TransparentRop(rop,tcolor); }
	void Visible(unsigned char vcolor) { VisibleRop(rop,vcolor); }
	void ForwardBackground(unsigned char fcolor) { ForwardBackgroundRop(rop,fcolor); }
	void Translucent(const RGBQUAD FAR * pal, unsigned char tc1, unsigned char tc2, int percent ) {
		TranslucentRop( pal, rop, tc1, tc2, percent) ;
	}
	void GrayScale( const RGBQUAD FAR * pal, int tcolor1 = 0, int tcolor2 = 255, unsigned char black = 0, unsigned char white = 255) {
		GrayScaleRop( pal, rop, tcolor1, tcolor2, black, white ) ;
	}
	void WriteToFile( LPCSTR filename ) const {
		WriteRopToFile( rop, filename ) ;
	}
	void ReadFromResource( HINSTANCE hInst, LPCSTR restype, LPCSTR resname ) {
		ReadRopFromResource( rop, hInst, restype, resname ) ;
	}
	void ReadFromResource( LPCSTR resname ) {
		ReadRopFromResource( rop, MainApp->getInstance(), RT_WINGLROP, resname ) ;
	}
	operator WINGLROP() const { return rop; }
} ;

class GTBL {
#if WINGL32
	BYTE tbl[256];
#else
	WINGLTBL tbl;
#endif
public:
#if WINGL32
	GTBL() { TblInit(tbl); }
	~GTBL() {}
#else
	GTBL() { tbl = CreateTbl(); }
	~GTBL() { DeleteTbl(tbl); }
#endif
	void GrayScale( const RGBQUAD FAR * pal, int tcolor1 = 0, int tcolor2 = 255, unsigned char black = 0, unsigned char white = 255) {
		TblGrayScale( pal, tbl, tcolor1, tcolor2, black, white ) ;
	}

	BYTE operator [] (int index) const { return tbl[(BYTE)index]; }
	operator WINGLTBL() { return (WINGLTBL)tbl; }
};

////////////////////////////////////////////////////////////////////////
//                              GMci
//
class GMci {
	union GMCI_PARAMS {
		MCI_PLAY_PARMS play;
		MCI_ANIM_RECT_PARMS animrect;
		MCI_ANIM_WINDOW_PARMS animwin;
		MCI_LOAD_PARMS load;
	};
	GMCI_PARAMS param;
	DWORD dwFlags;
	MCIDEVICEID deviceId;
	BOOL bEnable;
#if WINGL32
	DWORD threadID;
	BOOL bThread;
	BOOL bDone;
	DWORD dwResult;
	GMCI_PARAMS RunParam;
	DWORD dwRunFlags;
#endif
	MCI_STATUS_PARMS StatusParam;

public:
	GMci();
	~GMci();

	MCIDEVICEID GetDeviceID() const {
		return deviceId;
	}
	BOOL isOpen() const {
		return deviceId != 0;
	}

	void SetNotify(HWND hwnd) {
		param.play.dwCallback = (DWORD)hwnd;
		dwFlags |= MCI_NOTIFY;
	}
	void SetFrom(DWORD from) {
		param.play.dwFrom = from;
		dwFlags |= MCI_FROM;
	}

	void SetTo(DWORD to) {
		param.play.dwTo = to;
		dwFlags |= MCI_TO;
	}
	void SetWait() {
		dwFlags |= MCI_WAIT;
	}
	DWORD Open( LPCSTR szDevice, LPCSTR szModuleName ) ;
	DWORD Open( LPCSTR szModuleName ) {
		return Open( "sequencer", szModuleName ) ;
	}
	DWORD SendCommand(UINT uMsg) ;

	DWORD Load( LPCSTR szModuleName ) {
		dwFlags |= MCI_LOAD_FILE|MCI_WAIT;
		param.load.lpfilename = szModuleName;
		return SendCommand(MCI_LOAD);
	}
	DWORD Play() { return SendCommand(MCI_PLAY); }
	DWORD Pause() { return SendCommand(MCI_PAUSE); }
	DWORD Resume() { return SendCommand(MCI_RESUME); }
	DWORD Stop() { return SendCommand(MCI_STOP); }
	DWORD Seek(DWORD to) {
		param.play.dwFrom = to;
		dwFlags |= MCI_TO;
		return SendCommand(MCI_SEEK);
	}
	DWORD SetTimeFormat(DWORD fmt) {
		param.play.dwFrom = fmt;
		dwFlags |= MCI_SET_TIME_FORMAT|MCI_WAIT;
		return SendCommand(MCI_SET);
	}
	DWORD PutAnimRectDest( LPRECT rect ) {
		param.animrect.rc = *rect;
		dwFlags |= MCI_ANIM_RECT | MCI_ANIM_PUT_DESTINATION|MCI_WAIT;
		return SendCommand(MCI_PUT);
	}
	DWORD GetAnimRectSrc( LPRECT rect ) {
		dwFlags |= MCI_ANIM_WHERE_SOURCE|MCI_WAIT;
		DWORD a = SendCommand(MCI_WHERE);
		*rect = param.animrect.rc;
		return a;
	}
	DWORD SetAnimHWND( HWND hwnd ) {
		param.animwin.hWnd = hwnd;
		dwFlags |= MCI_ANIM_WINDOW_HWND|MCI_WAIT;
		return SendCommand(MCI_WINDOW);
	}
	DWORD RealizeAnim( BOOL bBackground ) {
		dwFlags |= bBackground ? (MCI_ANIM_REALIZE_BKGD|MCI_WAIT) : (MCI_ANIM_REALIZE_NORM|MCI_WAIT);
		return SendCommand(MCI_REALIZE);
	}
	DWORD Close() ;

	DWORD Status( DWORD dwFlags, DWORD dwItem, DWORD dwTrack = 0 ) ;

	DWORD GetLength() {
		return Status(MCI_STATUS_ITEM, MCI_STATUS_LENGTH);
	}
	DWORD GetPos() {
		return Status(MCI_STATUS_ITEM, MCI_STATUS_POSITION);
	}
	DWORD GetStartPos() {
		return Status(MCI_STATUS_ITEM | MCI_STATUS_START, MCI_STATUS_POSITION);
	}
	HPALETTE GetPalette() {
		return (HPALETTE)Status(MCI_STATUS_ITEM, MCI_ANIM_STATUS_HPAL );
	}

	// CDAudio specific
	//
	int GetNumTracks() {
		return (int)Status(MCI_STATUS_ITEM, MCI_STATUS_NUMBER_OF_TRACKS);
	}
	int GetCurrentTrack() {
		return (int)Status(MCI_STATUS_ITEM, MCI_STATUS_CURRENT_TRACK);
	}
	DWORD TrackPos( int track ) {
		return Status(MCI_STATUS_ITEM | MCI_TRACK, MCI_STATUS_POSITION, track);
	}

	//
	//
	BOOL isEnabled() const { return bEnable; }
	void Enable() { bEnable = TRUE; }
	void Disable() { Stop(); bEnable = FALSE; }

	DWORD _Open( LPCSTR szDevice, LPCSTR szModuleName ) ;
	DWORD _SendCommand(UINT uMsg) ;
	DWORD _Status( DWORD dwFlags );

#if WINGL32
private:
	static DWORD WINAPI MciThreadFunc(DWORD lpmci);
#endif
};

////////////////////////////////////////////////////////////////////////
//                              GWave
//
class GWave {
	BOOL bLoaded;
	int hwav;
	BOOL bEnable;
	static int UseCount;

public:
	GWave();
	~GWave();

	BOOL Open(HINSTANCE hInstApp, HWND hwnd, LPCSTR resname, BOOL is_file = FALSE, int max_simul = -1 );
	BOOL Open(HINSTANCE hInstApp, LPCSTR resname) {
		return Open(hInstApp, MainApp->GetMainWindow()->getHWND(), resname);
	}
	BOOL Open(LPCSTR resname, int max_simul = -1) {
		return Open(MainApp->getInstance(), MainApp->GetMainWindow()->getHWND(), resname, FALSE, max_simul );
	}
	BOOL OpenFile(LPCSTR filename, int max_simul = -1) {
		return Open(MainApp->getInstance(), MainApp->GetMainWindow()->getHWND(), filename, TRUE, max_simul );
	}
	void Close();
	int Play(DWORD flags = 0);
    BOOL SetPan(long lPan, int index = -1) {
		return bLoaded ? WinGLWavePanEx(hwav,lPan, index) : FALSE;
	}
    BOOL SetVolume(long lVolume, int index = -1) {
		return bLoaded ? WinGLWaveVolumeEx(hwav,lVolume, index) : FALSE;
	}
	void SetFrequency(int index, DWORD Hz);
	DWORD GetFrequency(int index) { return bLoaded ? WinGLWaveGetFrequency(hwav,index) : 0; }

	void Stop(int index = -1) { WinGLWaveStopEx(hwav, index); }

	static void Pump() { WinGLWavePump(); }
	static void Activate(BOOL b) { WinGLWaveActivate(b); }

	BOOL isEnabled() const { return bEnable; }
	void Enable() { bEnable = TRUE; }
	void Disable() { Stop(); bEnable = FALSE; }

	BOOL isOpen() const { return bLoaded; }
	BOOL isPlaying(int index = -1) { return bLoaded && IsWinGLWavePlayingEx(hwav, index); }
} ;

////////////////////////////////////////////////////////////////////////
//                              GInputObject
//
class GInputObject {
	HWINGLINPUT hkey;
public:
	enum KeyNum {
	  UpKey1 = WINGLKEY_UP1, UpKey2 = WINGLKEY_UP2, UpKey3 = WINGLKEY_UP3,
	  DownKey1 = WINGLKEY_DOWN1, DownKey2 = WINGLKEY_DOWN2, DownKey3 = WINGLKEY_DOWN3,
	  LeftKey1 = WINGLKEY_LEFT1, LeftKey2 = WINGLKEY_LEFT2, LeftKey3 = WINGLKEY_LEFT3,
	  RightKey1 = WINGLKEY_RIGHT1, RightKey2 = WINGLKEY_RIGHT2, RightKey3 = WINGLKEY_RIGHT3,
	  TriggerA1 = WINGLKEY_TRIGA1, TriggerA2 = WINGLKEY_TRIGA2,
	  TriggerB1 = WINGLKEY_TRIGB1, TriggerB2 = WINGLKEY_TRIGB2,
	  TriggerC1 = WINGLKEY_TRIGC1, TriggerC2 = WINGLKEY_TRIGC2,
	  TriggerD1 = WINGLKEY_TRIGD1, TriggerD2 = WINGLKEY_TRIGD2,
	  TriggerE1 = WINGLKEY_TRIGE1, TriggerE2 = WINGLKEY_TRIGE2,
	  TriggerF1 = WINGLKEY_TRIGF1, TriggerF2 = WINGLKEY_TRIGF2,
	  TriggerG1 = WINGLKEY_TRIGG1, TriggerG2 = WINGLKEY_TRIGG2,
	  TriggerH1 = WINGLKEY_TRIGH1, TriggerH2 = WINGLKEY_TRIGH2,
	  TriggerI1 = WINGLKEY_TRIGI1, TriggerI2 = WINGLKEY_TRIGI2,
	  TriggerJ1 = WINGLKEY_TRIGJ1, TriggerJ2 = WINGLKEY_TRIGJ2,
	};
	enum KeyHandle {
		Template1P = HWINGLKEY_1P,
		Template2P1 = HWINGLKEY_2P1,
		Template2P2 = HWINGLKEY_2P2,
	};

	GInputObject( enum KeyHandle handle) {
		hkey = (HWINGLINPUT)handle;
	}

	GInputObject() {
		hkey = WinGLCreateInput();
	}

	BOOL Assign( enum KeyNum keynum, WORD value ) const {
		return WinGLAssignInput(hkey, keynum, value);
	}
	WORD GetAssign( enum KeyNum keynum ) const {
		return (WORD)WinGLGetAssignInput(hkey, keynum);
	}
	BOOL AssignStick( WORD useStick ) const {
		return WinGLAssignStick(hkey, useStick);
	}
	WORD GetAssignStick() const {
		return WinGLGetAssignStick(hkey);
	}

	DWORD Capture(LPWINGLINPUTSTATE stat) const {
		return WinGLCaptureInput(hkey, stat);
	}
};

////////////////////////////////////////////////////////////////////////
//                              GInputState
//
class GInputState {
	WINGLINPUTSTATE stat;
public:
	GInputState() { 
#if WINGL16
	_fmemset( &stat, 0, sizeof stat);
#else
	memset( &stat, 0, sizeof stat);
#endif
	}
	GInputState( GInputObject& input ) { input.Capture(&stat); }

	void Capture( GInputObject& input ) {
		input.Capture( &stat );
	}

	BOOL up()    const { return (stat.press & STICK_UP) != 0; }
	BOOL down()  const { return (stat.press & STICK_DOWN) != 0; }
	BOOL right() const { return (stat.press & STICK_RIGHT) != 0; }
	BOOL left()  const { return (stat.press & STICK_LEFT) != 0; }

	BOOL trigger1() const { return (stat.press & STICK_TRIG1) != 0; }
	BOOL triggerA() const { return (stat.press & STICK_TRIG1) != 0; }
	BOOL trigger2() const { return (stat.press & STICK_TRIG2) != 0; }
	BOOL triggerB() const { return (stat.press & STICK_TRIG2) != 0; }
	BOOL trigger3() const { return (stat.press & STICK_TRIG3) != 0; }
	BOOL triggerC() const { return (stat.press & STICK_TRIG3) != 0; }
	BOOL trigger4() const { return (stat.press & STICK_TRIG4) != 0; }
	BOOL triggerD() const { return (stat.press & STICK_TRIG4) != 0; }
	BOOL trigger5() const { return (stat.press & STICK_TRIG5) != 0; }
	BOOL triggerE() const { return (stat.press & STICK_TRIG5) != 0; }
	BOOL trigger6() const { return (stat.press & STICK_TRIG6) != 0; }
	BOOL triggerF() const { return (stat.press & STICK_TRIG6) != 0; }
	BOOL trigger7() const { return (stat.press & STICK_TRIG7) != 0; }
	BOOL triggerG() const { return (stat.press & STICK_TRIG7) != 0; }
	BOOL trigger8() const { return (stat.press & STICK_TRIG8) != 0; }
	BOOL triggerH() const { return (stat.press & STICK_TRIG8) != 0; }
	BOOL trigger9() const { return (stat.press & STICK_TRIG9) != 0; }
	BOOL triggerI() const { return (stat.press & STICK_TRIG9) != 0; }
	BOOL trigger10() const { return (stat.press & STICK_TRIG10) != 0; }
	BOOL triggerJ() const { return (stat.press & STICK_TRIG10) != 0; }

	BOOL esc()  const { return (stat.press & STICK_ESC) != 0; }

	int x() const { return stat.x1; }
	int y() const { return stat.y1; }
	int x(int range) const { return stat.x1 * range / stat.x1max; }
	int y(int range) const { return stat.y1 * range / stat.x1max; }
	int x1() const { return stat.x1; }
	int y1() const { return stat.y1; }
	int x1(int range) const { return stat.x1 * range / stat.x1max; }
	int y1(int range) const { return stat.y1 * range / stat.x1max; }
	int x2() const { return stat.x2; }
	int y2() const { return stat.y2; }
	int x2(int range) const { return stat.x2 * range / stat.x1max; }
	int y2(int range) const { return stat.y2 * range / stat.x1max; }

	DWORD press() const { return stat.press; }
};

////////////////////////////////////////////////////////////////////////
//                              GScrollBar
//
#ifndef SIF_ALL

#define SIF_RANGE           0x0001
#define SIF_PAGE            0x0002
#define SIF_POS             0x0004
#define SIF_DISABLENOSCROLL 0x0008
#define SIF_TRACKPOS        0x0010
#define SIF_ALL             (SIF_RANGE | SIF_PAGE | SIF_POS | SIF_TRACKPOS)

struct SCROLLINFO {
	 UINT cbSize;
	 UINT fMask;
	 int  nMin;
	 int  nMax;
	 UINT nPage;
	 int  nPos;
	 int  nTrackPos;
};
typedef SCROLLINFO FAR *LPSCROLLINFO;
typedef const SCROLLINFO FAR *LPCSCROLLINFO;
#endif

typedef int     (WINAPI *PSetScrollInfo)(HWND, int, LPCSCROLLINFO, BOOL);
typedef BOOL    (WINAPI *PGetScrollInfo)(HWND, int, LPSCROLLINFO);

struct GScrollParam {
	int pos;
	int page;
	int min;
	int max;
};

class GScrollBar {
	HMODULE hm;
	PSetScrollInfo pSetScrollInfo;
	PGetScrollInfo pGetScrollInfo;

public:
	GScrollBar() {
		hm = LoadLibrary( "USER32.DLL" );
		pSetScrollInfo = (PSetScrollInfo)GetProcAddress(hm, "SetScrollInfo");
		pGetScrollInfo = (PGetScrollInfo)GetProcAddress(hm, "GetScrollInfo");
	}
	~GScrollBar() {
		FreeLibrary(hm);
	}

	// //
	void Set( HWND hwnd, int num,  GScrollParam * sp, BOOL fRedraw ) {
		if ( pSetScrollInfo ) {
			SCROLLINFO si;
			si.cbSize = sizeof si;
			si.fMask = SIF_PAGE|SIF_POS|SIF_RANGE;
			si.nPage = sp->page;
			si.nPos = sp->pos;
			si.nMin = sp->min;
			si.nMax = sp->max;
			pSetScrollInfo( hwnd, num, &si, fRedraw );
		} else {
			int min = sp->min;
			int max = sp->max;
			if ( max <= sp->page ) {
				min = max = 0;
			} else {
				max -= sp->page;
			}
			SetScrollRange( hwnd, num, min, max, FALSE );
			SetScrollPos( hwnd, num, sp->pos, fRedraw );
		}
	}

	int GetThumbTrack( HWND hwnd, int num, WPARAM 
#if WINGL32
												wparam
#endif
														, LPARAM
#if WINGL16
																	lparam
#endif
																			) const {
		if ( pGetScrollInfo ) {
			SCROLLINFO si;
			si.cbSize = sizeof si;
			si.fMask = SIF_POS|SIF_TRACKPOS;
			pGetScrollInfo( hwnd, num, &si );
			return si.nTrackPos;
		}
#if WINGL32
		return HIWORD(wparam);
#else
		return LOWORD(lparam);
#endif
	}

};
#endif	// _INC_GAME
